@@ -1,4 +1,4 @@
-# Copyright 2021 Good Chemistry Company.
+# Copyright 2023 Good Chemistry Company.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -20,9 +20,10 @@
 import unittest
 import os
 import time
+
 import numpy as np
 from openfermion.ops import QubitOperator
-from openfermion import load_operator
+from openfermion import load_operator, get_sparse_operator
 
 from tangelo.linq import Gate, Circuit, get_backend
 from tangelo.linq.translator import translate_circuit as translate_c
@@ -61,8 +62,9 @@ two_qubit_gates = [Gate(name, target=1, control=0) if name not in PARAMETERIZED_
 swap_gates = [Gate('SWAP', target=[1, 0]), Gate('CSWAP', target=[1, 2], control=0)]
 circuit5 = Circuit(init_gates + one_qubit_gates + two_qubit_gates + swap_gates)
 
-# Circuit preparing a mixed-state (e.g containing a MEASURE instruction in the middle of the circuit)
+# Circuit preparing a mixed-state (i.e. containing a MEASURE instruction in the middle of the circuit)
 circuit_mixed = Circuit([Gate("RX", 0, parameter=2.), Gate("RY", 1, parameter=-1.), Gate("MEASURE", 0), Gate("X", 0)])
+circuit_mixed_1 = Circuit([Gate("RX", 0, parameter=2.), Gate("RY", 0, parameter=-1.), Gate("MEASURE", 0), Gate("X", 0)])
 
 # Operators for testing the get_expectation_value functions
 op1 = 1.0 * QubitOperator('Z0')  # all Z
@@ -85,6 +87,8 @@ ref_freqs.append({'000': 0.15972060437359714, '100': 0.2828171838599203, '010':
 reference_exp_values = np.array([[0., 0., 0.], [0., -1., 0.], [-0.41614684, 0.7651474, -1.6096484], [1., 0., 0.],
                                  [-0.20175269, -0.0600213, 1.2972912]])
 reference_mixed = {'01': 0.163, '11': 0.066, '10': 0.225, '00': 0.545}  # With Qiskit noiseless, 1M shots
+reference_all = {'101': 0.163, '011': 0.066, '010': 0.225, '100': 0.545}
+reference_mid = {'1': 0.7, '0': 0.3}
 
 
 class TestSimulateAllBackends(unittest.TestCase):
@@ -112,12 +116,7 @@ class TestSimulateAllBackends(unittest.TestCase):
             self.assertTrue(exp_value == 777.)
 
     def test_simulate_mixed_state(self):
-        """ Test mid-circuit measurement (mixed-state simulation) for compatible/testable formats and backends.
-        Mixed-state do not have a statevector representation, as they are a statistical mixture of several statevectors.
-        Simulating individual shots is suitable.
-
-        Some simulators are NOT good at this, by design
-        """
+        """ Test mid-circuit measurement (mixed-state simulation) for compatible/testable formats and backends."""
 
         results = dict()
         for b in installed_simulator:
@@ -125,9 +124,34 @@ class TestSimulateAllBackends(unittest.TestCase):
             results[b], _ = sim.simulate(circuit_mixed)
             assert_freq_dict_almost_equal(results[b], reference_mixed, 1e-2)
 
+    def test_simulate_mixed_state_save_measures(self):
+        """ Test mid-circuit measurement (mixed-state simulation) for all installed backends."""
+        results = dict()
+        for b in installed_simulator:
+            sim = get_backend(target=b, n_shots=10 ** 3)
+            results[b], _ = sim.simulate(circuit_mixed, save_mid_circuit_meas=True)
+            assert_freq_dict_almost_equal(results[b], reference_mixed, 8e-2)
+            assert_freq_dict_almost_equal(sim.all_frequencies, reference_all, 8e-2)
+            assert_freq_dict_almost_equal(sim.mid_circuit_meas_freqs, reference_mid, 8e-2)
+
+    def test_simulate_mixed_state_desired_state(self):
+        """ Test mid-circuit measurement (mixed-state simulation) for all installed backends."""
+
+        results = dict()
+        exact = {'11': 0.23046888414227926, '10': 0.7695311158577207}
+        for b in installed_simulator:
+            sim = get_backend(target=b, n_shots=10 ** 3)
+            results[b], _ = sim.simulate(circuit_mixed, desired_meas_result="0")
+            assert_freq_dict_almost_equal(results[b], exact, 8.e-2)
+
+    def test_desired_meas_len(self):
+        """ Test if the desired_meas_result parameter is a string and of the right length."""
+        sim = get_backend(target="cirq", n_shots=10 ** 3)
+        self.assertRaises(ValueError, sim.simulate, circuit_mixed, desired_meas_result=0)
+        self.assertRaises(ValueError, sim.simulate, circuit_mixed, desired_meas_result="01")
+
     def test_get_exp_value_mixed_state(self):
-        """ Test expectation value for mixed-state simulation. Computation done by drawing individual shots.
-        Some simulators are NOT good at this, by design (ProjectQ). """
+        """ Test expectation value for mixed-state simulation. Computation done by drawing individual shots."""
 
         reference = 0.41614683  # Exact value
         results = dict()
@@ -191,6 +215,52 @@ class TestSimulateStatevector(unittest.TestCase):
                 frequencies, _ = simulator.simulate(circuit)
                 assert_freq_dict_almost_equal(ref_freqs[i], frequencies, atol=1e-5)
 
+    def test_simulate_mixed_state_desired_statevector(self):
+        """ Test mid-circuit measurement (mixed-state simulation) for compatible/testable formats and backends when returning
+        a statevector."""
+
+        results = dict()
+        results["qulacs"] = np.array([0. + 0.j, 0.87758256 + 0.j, 0. + 0.j, -0.47942554 + 0.j])
+        results["qiskit"] = np.array([0. + 0.j, 0.87758256 + 0.j, 0. + 0.j, -0.47942554 + 0.j])
+        results["cirq"] = np.array([0. + 0.j, 0. + 0.j, 0.87758256 + 0.j, -0.47942554 + 0.j])
+        initial_state = np.array([0, 0, 0, 1])
+        freqs_exact = {'10': 0.7701511529340699, '11': 0.2298488470659301}
+
+        for b in installed_sv_simulator:
+            sim = get_backend(target=b, n_shots=None)
+            f, sv = sim.simulate(circuit_mixed, desired_meas_result="0", return_statevector=True)
+            np.testing.assert_array_almost_equal(sv, results[b])
+            assert_freq_dict_almost_equal(f, freqs_exact, 1.e-7)
+
+            # Test that initial_statevector is respected
+            f, sv = sim.simulate(Circuit([Gate("MEASURE", 0), Gate("MEASURE", 1)]), desired_meas_result="11",
+                                 return_statevector=True, initial_statevector=initial_state)
+            np.testing.assert_array_almost_equal(sv, initial_state)
+            assert_freq_dict_almost_equal(f, {"11": 1}, 1.e-7)
+
+            # Test that ValueError is raised for desired_meas_result="0" with probability 0. i.e. loop exits successfully
+            self.assertRaises(ValueError, sim.simulate, Circuit([Gate("X", 0), Gate("MEASURE", 0)]), True, None, "0")
+
+            sim = get_backend(target=b, n_shots=10**3)
+            f, sv = sim.simulate(circuit_mixed, desired_meas_result="0", return_statevector=True)
+            np.testing.assert_array_almost_equal(sv, results[b])
+            assert_freq_dict_almost_equal(f, freqs_exact, 1.e-1)
+
+    def test_mixed_state_save_measures_return_statevector(self):
+        """ Test functionality to return statevector if mid-circuit measurement is saved and n_shots must be 1"""
+
+        sv_exact = {"0": np.array([0.+0.j, 0.76163265-0.64800903j]),
+                    "1": np.array([-0.33100336-0.94362958j, 0.+0.j])}
+        for b in installed_sv_simulator:
+            sim = get_backend(target=b, n_shots=1)
+            _, sv = sim.simulate(circuit_mixed_1, save_mid_circuit_meas=True, return_statevector=True)
+            np.testing.assert_array_almost_equal(sv, sv_exact[next(iter(sim.mid_circuit_meas_freqs))])
+
+        # Assert raises error when return_statevector=True and n_shots != 1
+        for b in installed_sv_simulator:
+            sim = get_backend(target=b, n_shots=2)
+            self.assertRaises(ValueError, sim.simulate, circuit_mixed_1, True, None, None, True)
+
     def test_simulate_nshots_from_statevector(self):
         """
             Test the generation of samples following the distribution given by the exact frequencies obtained
@@ -348,6 +418,18 @@ class TestSimulateStatevector(unittest.TestCase):
         energy = simulator.get_expectation_value(qubit_operator, abs_circ)
         self.assertAlmostEqual(energy, expected, delta=1e-3)
 
+    def test_get_exp_value_mixed_state_desired_measurement_with_shots(self):
+        """ Get expectation value of mixed state by post-selecting on desired measurement."""
+        qubit_operator = QubitOperator("X0 X1") + QubitOperator("Y0 Y1") + QubitOperator("Z0 Z1") + QubitOperator("X0 Y1", 1j)
+
+        ham = get_sparse_operator(qubit_operator).toarray()
+        exact_sv = np.array([0.+0.j, 0.+0.j, 0.87758256+0.j, -0.47942554+0.j])
+        exact_exp = np.vdot(exact_sv, ham @ exact_sv)
+
+        simulator = get_backend(n_shots=10**4)
+        sim_exp = simulator.get_expectation_value(qubit_operator, circuit_mixed, desired_meas_result="0")
+        self.assertAlmostEqual(exact_exp, sim_exp, delta=1.e-1)
+
     def test_get_exp_value_empty_circuit(self):
         """ If the circuit is empty and we have a non-zero number of qubits, frequencies just only show all-|0> state
         observed and compute the expectation value using these frequencies """
