@@ -1,4 +1,4 @@
-# Copyright 2021 Good Chemistry Company.
+# Copyright 2023 Good Chemistry Company.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -24,6 +24,7 @@ from .translate_projectq import translate_projectq, _translate_projectq2abs, get
 from .translate_openqasm import translate_openqasm, _translate_openqasm2abs, get_openqasm_gates
 from .translate_qubitop import translate_operator
 from .translate_circuit import translate_circuit
+from .translate_pennylane import get_pennylane_gates
 
 
 def get_supported_gates():
@@ -38,5 +39,6 @@ def get_supported_gates():
     supported_gates["qiskit"] = sorted(get_qiskit_gates().keys())
     supported_gates["cirq"] = sorted(get_cirq_gates().keys())
     supported_gates["braket"] = sorted(get_braket_gates().keys())
+    supported_gates["pennylane"] = sorted(get_pennylane_gates().keys())
 
     return supported_gates
