@@ -1,4 +1,4 @@
-# Copyright 2021 Good Chemistry Company.
+# Copyright 2023 Good Chemistry Company.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -24,6 +24,7 @@ necessary to account for:
 
 from math import pi
 
+from tangelo.toolboxes.operators import QubitOperator
 from tangelo.helpers import deprecated
 
 
@@ -76,12 +77,16 @@ def translate_cirq(source_circuit):
     return translate_c_to_cirq(source_circuit)
 
 
-def translate_c_to_cirq(source_circuit, noise_model=None):
+def translate_c_to_cirq(source_circuit, noise_model=None, save_measurements=False):
     """Take in an abstract circuit, return an equivalent cirq QuantumCircuit
     object.
 
     Args:
-        source_circuit: quantum circuit in the abstract format.
+        source_circuit (Circuit): quantum circuit in the abstract format.
+        noise_model (NoiseModel): The noise model to use
+        save_measurements (bool): If True, all measurements in the circuit are saved
+            with the key 'n' for the nth measurement in the Circuit. If False, no
+            measurements are saved.
 
     Returns:
         cirq.Circuit: a corresponding cirq Circuit. Right now, the structure is
@@ -99,6 +104,8 @@ def translate_c_to_cirq(source_circuit, noise_model=None):
     # cirq will otherwise only initialize qubits that have gates
     target_circuit.append(cirq.I.on_each(qubit_list))
 
+    measure_count = 0
+
     # Maps the gate information properly. Different for each backend (order, values)
     for gate in source_circuit._gates:
         if (gate.control is not None) and gate.name != 'CNOT':
@@ -115,7 +122,9 @@ def translate_c_to_cirq(source_circuit, noise_model=None):
         elif gate.name in {"CNOT"}:
             target_circuit.append(GATE_CIRQ[gate.name](qubit_list[gate.control[0]], qubit_list[gate.target[0]]))
         elif gate.name in {"MEASURE"}:
-            target_circuit.append(GATE_CIRQ[gate.name](qubit_list[gate.target[0]]))
+            key = str(measure_count) if save_measurements else None
+            target_circuit.append(GATE_CIRQ[gate.name](qubit_list[gate.target[0]], key=key))
+            measure_count += 1
         elif gate.name in {"CRZ", "CRX", "CRY"}:
             next_gate = GATE_CIRQ[gate.name](gate.parameter).controlled(num_controls)
             target_circuit.append(next_gate(*control_list, qubit_list[gate.target[0]]))
@@ -156,3 +165,45 @@ def translate_c_to_cirq(source_circuit, noise_model=None):
                     target_circuit.append(depo(*depo_list))  # gates targeted
 
     return target_circuit
+
+
+def translate_op_to_cirq(qubit_operator):
+    """Helper function to translate a Tangelo QubitOperator to a cirq linear
+    combination of Pauli strings.
+
+    Args:
+        qubit_operator (tangelo.toolboxes.operators.QubitOperator): Self-explanatory.
+
+    Returns:
+        (cirq.ops.linear_combinations.PauliSum): Cirq summation of Pauli strings.
+    """
+    from openfermion.transforms import qubit_operator_to_pauli_sum
+
+    return qubit_operator_to_pauli_sum(qubit_operator)
+
+
+def translate_op_from_cirq(qubit_operator):
+    """Helper function to translate a cirq linear combination of Pauli strings
+    to a Tangelo QubitOperator.
+
+    Args:
+        qubit_operator (cirq.ops.linear_combinations.PauliSum): Self-explanatory.
+
+    Returns:
+        (tangelo.toolboxes.operators.QubitOperator): Tangelo qubit operator.
+    """
+    import cirq
+
+    tangelo_op = QubitOperator()
+
+    cirq_pauli_to_string = {cirq.X: "X", cirq.Y: "Y", cirq.Z: "Z"}
+
+    for pauli_word in qubit_operator:
+
+        term_string = ""
+        for line_qubit, cirq_pauli in pauli_word.items():
+            term_string += f"{cirq_pauli_to_string[cirq_pauli]}{line_qubit.x} "
+
+        tangelo_op += QubitOperator(term_string.strip(), pauli_word.coefficient)
+
+    return tangelo_op
