@@ -1,4 +1,4 @@
-# Copyright 2021 Good Chemistry Company.
+# Copyright 2023 Good Chemistry Company.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -64,7 +64,7 @@ def translate_qsharp(source_circuit):
     return translate_c_to_qsharp(source_circuit)
 
 
-def translate_c_to_qsharp(source_circuit, operation="MyQsharpOperation"):
+def translate_c_to_qsharp(source_circuit, operation="MyQsharpOperation", save_measurements=False):
     """Take in an abstract circuit, generate the corresponding Q# operation
     (state prep + measurement) string, in the appropriate Q# template. The Q#
     output can be written to file and will be compiled at runtime.
@@ -72,6 +72,9 @@ def translate_c_to_qsharp(source_circuit, operation="MyQsharpOperation"):
     Args:
         source_circuit: quantum circuit in the abstract format.
         operation (str), optional: name of the Q# operation.
+        save_measurements (bool), optional: True, return all mid-circuit measurement results.
+            This returns a frequency vector that is of size 2^(n_meas+n_qubits). False,
+            all measurements are overwritten.
 
     Returns:
         str: The Q# code (operation + template). This needs to be written into a
@@ -80,12 +83,15 @@ def translate_c_to_qsharp(source_circuit, operation="MyQsharpOperation"):
 
     GATE_QDK = get_qdk_gates()
 
+    n_meas = source_circuit._gate_counts.get("MEASURE", 0) if save_measurements else 0
+    n_c = n_meas + source_circuit.width
+    measurement = 0
     # Prepare Q# operation header
     qsharp_string = ""
     qsharp_string += "@EntryPoint()\n"
     qsharp_string += f"operation {operation}() : Result[] {{\n"
 #    qsharp_string += "body (...) {\n\n"
-    qsharp_string += f"\tmutable c = new Result[{source_circuit.width}];\n"
+    qsharp_string += f"\tmutable c = new Result[{n_c}];\n"
     qsharp_string += f"\tusing (qreg = Qubit[{source_circuit.width}]) {{\n"
 
     # Generate Q# strings with the right syntax, order and values for the gate inputs
@@ -112,10 +118,13 @@ def translate_c_to_qsharp(source_circuit, operation="MyQsharpOperation"):
         elif gate.name in {"CSWAP"}:
             body_str += f"\t\tControlled {GATE_QDK[gate.name]}({control_string}, (qreg[{gate.target[0]}], qreg[{gate.target[1]}]));\n"
         elif gate.name in {"MEASURE"}:
-            body_str += f"\t\tset c w/= {gate.target[0]} <- {GATE_QDK[gate.name]}(qreg[{gate.target[0]}]);\n"
+            body_str += f"\t\tset c w/= {measurement} <- {GATE_QDK[gate.name]}(qreg[{gate.target[0]}]);\n"
+            if save_measurements:
+                measurement += 1
         else:
             raise ValueError(f"Gate '{gate.name}' not supported on backend qdk")
-    qsharp_string += body_str + "\n\t\treturn ForEach(MResetZ, qreg);\n"
+    return_str = f"\n\t\tfor index in 0 .. Length(qreg) - 1 {{\n\t\t\tset c w/= {n_meas} + index <- MResetZ(qreg[index]);\n\t\t}}\n"
+    qsharp_string += body_str + return_str + "\n\t\treturn c;\n"
     qsharp_string += "\t}\n"
 #    qsharp_string += "}\n adjoint auto;\n"
     qsharp_string += "}\n"
