@@ -1,4 +1,4 @@
-# Copyright 2021 Good Chemistry Company.
+# Copyright 2023 Good Chemistry Company.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -79,11 +79,13 @@ def translate_qiskit(source_circuit):
     return translate_c_to_qiskit(source_circuit)
 
 
-def translate_c_to_qiskit(source_circuit: Circuit):
+def translate_c_to_qiskit(source_circuit: Circuit, save_measurements=False):
     """Take in a Circuit, return an equivalent qiskit.QuantumCircuit
 
     Args:
-        source_circuit (Circuit): quantum circuit in the Tangelo format.
+        source_circuit (Circuit): quantum circuit in the abstract format.
+        save_measurements (bool): Return mid-circuit measurements in the order
+            they appear in the circuit in the classical registers
 
     Returns:
         qiskit.QuantumCircuit: the corresponding qiskit.QuantumCircuit
@@ -92,7 +94,11 @@ def translate_c_to_qiskit(source_circuit: Circuit):
 
     GATE_QISKIT = get_qiskit_gates()
 
-    target_circuit = qiskit.QuantumCircuit(source_circuit.width, source_circuit.width)
+    n_meas = source_circuit._gate_counts.get("MEASURE", 0) if save_measurements else 0
+    n_measures = n_meas + source_circuit.width
+    target_circuit = qiskit.QuantumCircuit(source_circuit.width, n_measures)
+
+    measurement = 0
 
     # Maps the gate information properly. Different for each backend (order, values)
     for gate in source_circuit._gates:
@@ -114,7 +120,9 @@ def translate_c_to_qiskit(source_circuit: Circuit):
         elif gate.name in {"XX"}:
             (GATE_QISKIT[gate.name])(target_circuit, gate.parameter, gate.target[0], gate.target[1])
         elif gate.name in {"MEASURE"}:
-            (GATE_QISKIT[gate.name])(target_circuit, gate.target[0], gate.target[0])
+            (GATE_QISKIT[gate.name])(target_circuit, gate.target[0], measurement)
+            if save_measurements:
+                measurement += 1
         else:
             raise ValueError(f"Gate '{gate.name}' not supported on backend qiskit")
 
@@ -205,13 +213,10 @@ def translate_op_from_qiskit(qubit_operator):
         # Inversion of the string because of qiskit ordering.
         term_string = pauli_word.to_pauli_op().primitive.to_label()[::-1]
         term_tuple = pauli_string_to_of(term_string)
-        terms_dict[tuple(term_tuple)] = pauli_word.coeff
+        terms_dict[tuple(term_tuple)] = complex(pauli_word.coeffs)
 
     # Create and copy the information into a new QubitOperator.
     tangelo_op = QubitOperator()
     tangelo_op.terms = terms_dict
 
-    # Clean the QubitOperator.
-    tangelo_op.compress()
-
     return tangelo_op
