@@ -1,4 +1,4 @@
-# Copyright 2021 Good Chemistry Company.
+# Copyright 2023 Good Chemistry Company.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -23,6 +23,8 @@ necessary to account for:
 """
 
 from numpy import exp, cos, sin
+
+from tangelo.toolboxes.operators import QubitOperator
 from tangelo.helpers import deprecated
 
 
@@ -78,16 +80,19 @@ def translate_qulacs(source_circuit, noise_model=None):
     return translate_c_to_qulacs(source_circuit, noise_model)
 
 
-def translate_c_to_qulacs(source_circuit, noise_model=None):
+def translate_c_to_qulacs(source_circuit, noise_model=None, save_measurements=False):
     """Take in an abstract circuit, return an equivalent qulacs QuantumCircuit
     instance. If provided with a noise model, will add noisy gates at
     translation. Not very useful to look at, as qulacs does not provide much
     information about the noisy gates added when printing the "noisy circuit".
 
     Args:
-        source_circuit: quantum circuit in the abstract format.
-        noise_model: A NoiseModel object from this package, located in the
+        source_circuit (Circuit): quantum circuit in the abstract format.
+        noise_model (NoiseModel): A NoiseModel object from this package, located in the
             noisy_simulation subpackage.
+        save_measurements (bool): If True, each nth measurement in the circuit is saved in
+            the nth classical register. Otherwise, each measurement overwrites
+            the first classical register.
 
     Returns:
         qulacs.QuantumCircuit: the corresponding qulacs quantum circuit.
@@ -99,6 +104,8 @@ def translate_c_to_qulacs(source_circuit, noise_model=None):
     GATE_QULACS = get_qulacs_gates()
     target_circuit = qulacs.QuantumCircuit(source_circuit.width)
 
+    measure_count = 0
+
     # Maps the gate information properly. Different for each backend (order, values)
     for gate in source_circuit._gates:
         if gate.name in {"H", "X", "Y", "Z", "S", "T"}:
@@ -141,8 +148,10 @@ def translate_c_to_qulacs(source_circuit, noise_model=None):
         elif gate.name in {"CNOT"}:
             (GATE_QULACS[gate.name])(target_circuit, gate.control[0], gate.target[0])
         elif gate.name in {"MEASURE"}:
-            gate = (GATE_QULACS[gate.name])(gate.target[0], gate.target[0])
-            target_circuit.add_gate(gate)
+            m_gate = (GATE_QULACS[gate.name])(gate.target[0], measure_count)
+            target_circuit.add_gate(m_gate)
+            if save_measurements:
+                measure_count += 1
         else:
             raise ValueError(f"Gate '{gate.name}' not supported on backend qulacs")
 
@@ -168,3 +177,47 @@ def translate_c_to_qulacs(source_circuit, noise_model=None):
                         raise ValueError(f'{gate.name} has more than 2 qubits, Qulacs DepolarizingNoise only supports 1- and 2-qubits')
 
     return target_circuit
+
+
+def translate_op_to_qulacs(qubit_operator):
+    """Helper function to translate a Tangelo QubitOperator to a qulacs general
+    quantum operator.
+
+    Args:
+        qubit_operator (tangelo.toolboxes.operators.QubitOperator): Self-explanatory.
+
+    Returns:
+        (qulacs_core.GeneralQuantumOperator): Qulacs quantum operator.
+    """
+    from qulacs.quantum_operator import create_quantum_operator_from_openfermion_text
+
+    return create_quantum_operator_from_openfermion_text(qubit_operator.__repr__())
+
+
+def translate_op_from_qulacs(qubit_operator):
+    """Helper function to translate a qulacs general quantum operator to a
+    Tangelo QubitOperator.
+
+    Args:
+        qubit_operator (qulacs_core.GeneralQuantumOperator): Self-explanatory.
+
+    Returns:
+        (tangelo.toolboxes.operators.QubitOperator): Tangelo qubit operator.
+    """
+
+    tangelo_op = QubitOperator()
+
+    # Not considering 0: "I", because it is not a valid action in
+    # openfermion-like qubit operators.
+    qulacs_pauli_id_to_string = {1: "X", 2: "Y", 3: "Z"}
+
+    n_terms = qubit_operator.get_term_count()
+    for term_i in range(n_terms):
+        pauli_ids = qubit_operator.get_term(term_i).get_pauli_id_list()
+        pauli_word = [qulacs_pauli_id_to_string[i] for i in pauli_ids if i != 0]
+        qubit_indices = qubit_operator.get_term(term_i).get_index_list()
+        pauli_term = tuple(zip(qubit_indices, pauli_word))
+
+        tangelo_op += QubitOperator(pauli_term, qubit_operator.get_term(term_i).get_coef())
+
+    return tangelo_op
