@@ -1,4 +1,4 @@
-# Copyright 2021 Good Chemistry Company.
+# Copyright 2023 Good Chemistry Company.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -12,10 +12,16 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
+# Common helper functions for all mean-field.
 from .dmet_orbitals import dmet_orbitals as _orbitals
 from .dmet_fragment import dmet_fragment_constructor as _fragment_constructor
-from .dmet_onerdm import dmet_low_rdm as _low_rdm
 from .dmet_onerdm import dmet_fragment_rdm as _fragment_rdm
 from .dmet_bath import dmet_fragment_bath as _fragment_bath
-from .dmet_scf_guess import dmet_fragment_guess as _fragment_guess
-from .dmet_scf import dmet_fragment_scf as _fragment_scf
+
+# Specific helper functions for restricted / unrestricted mean-field.
+from .dmet_onerdm import dmet_low_rdm_rhf as _low_rdm_rhf
+from .dmet_onerdm import dmet_low_rdm_rohf_uhf as _low_rdm_rohf_uhf
+from .dmet_scf_guess import dmet_fragment_guess_rhf as _fragment_guess_rhf
+from .dmet_scf_guess import dmet_fragment_guess_rohf_uhf as _fragment_guess_rohf_uhf
+from .dmet_scf import dmet_fragment_scf_rhf as _fragment_scf_rhf
+from .dmet_scf import dmet_fragment_scf_rohf_uhf as _fragment_scf_rohf_uhf
