@@ -1,4 +1,4 @@
-# Copyright 2021 Good Chemistry Company.
+# Copyright 2023 Good Chemistry Company.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -15,7 +15,6 @@
 """Employ DMET as a problem decomposition technique."""
 
 from enum import Enum
-from functools import reduce
 import numpy as np
 from pyscf import gto, scf
 import scipy
@@ -23,16 +22,18 @@ import warnings
 
 from tangelo.problem_decomposition.dmet import _helpers as helpers
 from tangelo.problem_decomposition.problem_decomposition import ProblemDecomposition
-from tangelo.problem_decomposition.electron_localization import iao_localization, meta_lowdin_localization
+from tangelo.problem_decomposition.electron_localization import iao_localization, meta_lowdin_localization, nao_localization
 from tangelo.problem_decomposition.dmet.fragment import SecondQuantizedDMETFragment
 from tangelo.algorithms import FCISolver, CCSDSolver, VQESolver
 from tangelo.toolboxes.post_processing.mc_weeny_rdm_purification import mcweeny_purify_2rdm
+from tangelo.toolboxes.molecular_computation.rdms import pad_rdms_with_frozen_orbitals
 
 
 class Localization(Enum):
     """Enumeration of the electron localization supported by DMET."""
     meta_lowdin = 0
     iao = 1
+    nao = 2
 
 
 class DMETProblemDecomposition(ProblemDecomposition):
@@ -43,6 +44,10 @@ class DMETProblemDecomposition(ProblemDecomposition):
     used instead of the Meta-Lowdin localization scheme, but it cannot be used
     for minimal basis set.
 
+    The underlying mean-field for the computation is automatically detected
+    from the SecondQuantizedMolecule. RHF, ROHF and UHF mean-fields are
+    supported.
+
     Attributes:
         molecule (SecondQuantizedMolecule): The molecular system.
         electron_localization (Localization): A type of localization scheme.
@@ -52,6 +57,9 @@ class DMETProblemDecomposition(ProblemDecomposition):
             system.
         fragment_solvers (list or str): List of solvers for each fragment. If
             only a string is detected, this solver is used for all fragments.
+        fragment_frozen_orbitals (list): List of frozen orbitals. List of
+            integers freezes the n first orbitals in the fragment. Nested list
+            of integers freezes orbitals based on indices.
         optimizer (function handle): A function defining the classical optimizer
             and its behavior.
         initial_chemical_potential (float) : Initial value for the chemical
@@ -75,6 +83,7 @@ class DMETProblemDecomposition(ProblemDecomposition):
                            "electron_localization": Localization.meta_lowdin,
                            "fragment_atoms": list(),
                            "fragment_solvers": "ccsd",
+                           "fragment_frozen_orbitals": list(),
                            "optimizer": self._default_optimizer,
                            "initial_chemical_potential": 0.0,
                            "solvers_options": list(),
@@ -95,6 +104,9 @@ class DMETProblemDecomposition(ProblemDecomposition):
         if not self.molecule:
             raise ValueError(f"A SecondQuantizedMolecule object must be provided when instantiating DMETProblemDecomposition.")
 
+        if self.molecule.uhf or self.molecule.spin != 0:
+            raise NotImplementedError("ROHF-DMET and UHF-DMET have been disabled temporarily for code validation purposes.")
+
         # Converting our interface to pyscf.mol.gto and pyscf.scf (used by this
         # code).
         self.mean_field = self.molecule.mean_field
@@ -139,6 +151,11 @@ class DMETProblemDecomposition(ProblemDecomposition):
         if self.molecule.natm != sum(self.fragment_atoms):
             raise RuntimeError("The number of fragment sites is not equal to the number of atoms in the molecule")
 
+        if not len(self.fragment_frozen_orbitals):
+            self.fragment_frozen_orbitals = [None] * len(self.fragment_atoms)
+        elif len(self.fragment_frozen_orbitals) != len(self.fragment_atoms):
+            raise RuntimeError("The number of elements in the frozen orbitals list does not match the number of fragments.")
+
         # Check that the number of solvers matches the number of fragments.
         # If a single string is detected, it is converted to a list.
         # If a list is detected, it must have the same length than the fragment_atoms.
@@ -176,6 +193,13 @@ class DMETProblemDecomposition(ProblemDecomposition):
         self.orb_list2 = None
         self.onerdm_low = None
 
+        # If save_results in _oneshot_loop is True, the dict is populated.
+        self.solver_fragment_dict = dict()
+
+        # To keep track the number of iteration (was done with an energy list
+        # before).
+        self.n_iter = 0
+
     @property
     def quantum_fragments_data(self):
         """This aims to return a dictionary with all necessary components to
@@ -206,19 +230,21 @@ class DMETProblemDecomposition(ProblemDecomposition):
                 self.electron_localization = meta_lowdin_localization
             elif self.electron_localization == Localization.iao:
                 self.electron_localization = iao_localization
+            elif self.electron_localization == Localization.nao:
+                self.electron_localization = nao_localization
             else:
                 raise ValueError(f"Unsupported ansatz. Built-in localization methods:\n\t{self.builtin_localization}")
         elif not callable(self.electron_localization):
             raise TypeError(f"Invalid electron localization function. Expecting a function.")
 
         # Construct orbital object.
-        self.orbitals = helpers._orbitals(self.molecule, self.mean_field, range(self.molecule.nao_nr()), self.electron_localization)
+        self.orbitals = helpers._orbitals(self.molecule, self.mean_field, range(self.molecule.nao_nr()), self.electron_localization, False)
 
         # TODO: remove last argument, combining fragments not supported.
         self.orb_list, self.orb_list2, _ = helpers._fragment_constructor(self.molecule, self.fragment_atoms, 0)
 
         # Calculate the 1-RDM for the entire molecule.
-        self.onerdm_low = helpers._low_rdm(self.orbitals.active_fock, self.orbitals.number_active_electrons)
+        self.onerdm_low = helpers._low_rdm_rhf(self.orbitals.active_fock, self.orbitals.number_active_electrons)
 
     def simulate(self):
         """Perform DMET loop to optimize the chemical potential. It converges
@@ -229,10 +255,6 @@ class DMETProblemDecomposition(ProblemDecomposition):
             float: The DMET energy (dmet_energy).
         """
 
-        # To keep track the number of iteration (was done with an energy list before).
-        # TODO: A decorator function to do the same thing?
-        self.n_iter = 0
-
         # Initialize the energy list and SCF procedure employing newton-raphson algorithm.
         # TODO : find a better initial guess than 0.0 for chemical potential. DMET fails often currently.
         if not self.orbitals:
@@ -326,12 +348,12 @@ class DMETProblemDecomposition(ProblemDecomposition):
             one_ele, fock, two_ele = self.orbitals.dmet_fragment_hamiltonian(bath_orb, norb_high, onerdm_high)
 
             # Construct guess orbitals for fragment SCF calculations.
-            guess_orbitals = helpers._fragment_guess(t_list, bath_orb, chemical_potential, norb_high, nelec_high,
-                                                     self.orbitals.active_fock)
-
             # Carry out SCF calculation for a fragment.
-            mf_fragment, fock_frag_copy, mol_frag = helpers._fragment_scf(t_list, two_ele, fock, nelec_high, norb_high,
-                                                                          guess_orbitals, chemical_potential)
+            guess_orbitals = helpers._fragment_guess_rhf(t_list, bath_orb, chemical_potential, norb_high, nelec_high,
+                                                            self.orbitals.active_fock)
+            mf_fragment, fock_frag_copy, mol_frag = helpers._fragment_scf_rhf(
+                t_list, two_ele, fock, nelec_high, norb_high, guess_orbitals,
+                chemical_potential)
 
             scf_fragments.append([mf_fragment, fock_frag_copy, mol_frag, t_list, one_ele, two_ele, fock])
 
@@ -370,7 +392,6 @@ class DMETProblemDecomposition(ProblemDecomposition):
         Returns:
             float: The new chemical potential.
         """
-
         self.n_iter += 1
         if self.verbose:
             print(" \tIteration = ", self.n_iter)
@@ -408,7 +429,9 @@ class DMETProblemDecomposition(ProblemDecomposition):
             # We create a dummy SecondQuantizedMolecule with a DMETFragment class.
             # It has the same important attributes and methods to be used with
             # functions of this package.
-            dummy_mol = SecondQuantizedDMETFragment(mol_frag, mf_fragment, fock, fock_frag_copy, t_list, one_ele, two_ele)
+            dummy_mol = SecondQuantizedDMETFragment(mol_frag, mf_fragment, fock,
+                fock_frag_copy, t_list, one_ele, two_ele, False,
+                self.fragment_frozen_orbitals[i])
 
             if self.verbose:
                 print("\t\tFragment Number : # ", i + 1)
@@ -455,20 +478,18 @@ class DMETProblemDecomposition(ProblemDecomposition):
                     self.solver_fragment_dict[i] = solver_fragment
                     self.rdm_measurements[i] = self.solver_fragment_dict[i].rdm_freq_dict
 
-            fragment_energy, _, one_rdm = self._compute_energy(mf_fragment, onerdm, twordm,
-                                                               fock_frag_copy, t_list, one_ele,
-                                                               two_ele, fock)
+            # Compute the fragment energy and sum up the number of electrons
+            onerdm_padded, twordm_padded = pad_rdms_with_frozen_orbitals(dummy_mol, onerdm, twordm)
+            fragment_energy, onerdm = self._compute_energy_restricted(dummy_mol, onerdm_padded, twordm_padded)
+            n_electron_frag = np.trace(onerdm[: t_list[0], : t_list[0]])
+
+            number_of_electron += n_electron_frag
 
             # Sum up the energy.
             energy_temp += fragment_energy
 
-            # Sum up the number of electrons.
-            number_of_electron += np.trace(one_rdm[: t_list[0], : t_list[0]])
-
             if self.verbose:
-                print("\t\tFragment Energy                 = " + "{:17.10f}".format(fragment_energy))
-                print("\t\tNumber of Electrons in Fragment = " + "{:17.10f}".format(np.trace(one_rdm)))
-                print("")
+                print(f"\t\tFragment Energy = {fragment_energy}\n\t\tNumber of Electrons in Fragment = {n_electron_frag}")
 
         energy_temp += self.orbitals.core_constant_energy
         self.dmet_energy = energy_temp.real
@@ -497,7 +518,9 @@ class DMETProblemDecomposition(ProblemDecomposition):
             # Unpacking the information for the selected fragment.
             mf_fragment, fock_frag_copy, mol_frag, t_list, one_ele, two_ele, fock = info_fragment
 
-            dummy_mol = SecondQuantizedDMETFragment(mol_frag, mf_fragment, fock, fock_frag_copy, t_list, one_ele, two_ele)
+            dummy_mol = SecondQuantizedDMETFragment(mol_frag, mf_fragment, fock,
+                fock_frag_copy, t_list, one_ele, two_ele, False,
+                self.fragment_frozen_orbitals[i])
 
             # Buiding SCF fragments and quantum circuit. Resources are then
             # estimated. For classical sovlers, this functionality is not
@@ -518,53 +541,123 @@ class DMETProblemDecomposition(ProblemDecomposition):
 
         return resources_fragments
 
-    def _compute_energy(self, mf_frag, onerdm, twordm, fock_frag_copy, t_list, oneint, twoint, fock):
+    def _compute_energy_restricted(self, dmet_fragment, onerdm, twordm):
         """Calculate the fragment energy.
 
         Args:
-            mf_frag (pyscf.scf.RHF): The mean field of the fragment.
+            dmet_fragment (SecondQuantizedDMETFragment): Self-explanatory.
             onerdm (numpy.array): one-particle reduced density matrix (float).
             twordm (numpy.array): two-particle reduced density matrix (float).
-            fock_frag_copy (numpy.array): Fock matrix with the chemical
-                potential subtracted (float).
-            t_list (list): List of number of fragment and bath orbitals (int).
-            oneint (numpy.array): One-electron integrals of fragment (float).
-            twoint (numpy.array): Two-electron integrals of fragment (float).
-            fock (numpy.array): Fock matrix of fragment (float).
 
         Returns:
             float: Fragment energy (fragment_energy).
-            float: Total energy for fragment using RDMs (total_energy_rdm).
-            numpy.array: One-particle RDM for a fragment (one_rdm, float).
+            numpy.array: One-particle RDM for a fragment (onerdm, float).
         """
 
-        # Execute CCSD calculation
+        fock = dmet_fragment.fock
+        t_list = dmet_fragment.t_list
+        oneint = dmet_fragment.one_ele
+        twoint = dmet_fragment.two_ele
+        mo_coeff = dmet_fragment.mean_field.mo_coeff
+
         norb = t_list[0]
 
         # Calculate the one- and two- RDM for DMET energy calculation (Transform to AO basis)
-        one_rdm = reduce(np.dot, (mf_frag.mo_coeff, onerdm, mf_frag.mo_coeff.T))
+        onerdm = mo_coeff @ onerdm @ mo_coeff.T
 
-        twordm = np.einsum("pi,ijkl->pjkl", mf_frag.mo_coeff, twordm)
-        twordm = np.einsum("qj,pjkl->pqkl", mf_frag.mo_coeff, twordm)
-        twordm = np.einsum("rk,pqkl->pqrl", mf_frag.mo_coeff, twordm)
-        twordm = np.einsum("sl,pqrl->pqrs", mf_frag.mo_coeff, twordm)
-
-        # Calculate the total energy based on RDMs
-        total_energy_rdm = np.einsum("ij,ij->", fock_frag_copy, one_rdm) + 0.5 * np.einsum("ijkl,ijkl->", twoint,
-                                                                                           twordm)
+        twordm = np.einsum("pi,ijkl->pjkl", mo_coeff, twordm)
+        twordm = np.einsum("qj,pjkl->pqkl", mo_coeff, twordm)
+        twordm = np.einsum("rk,pqkl->pqrl", mo_coeff, twordm)
+        twordm = np.einsum("sl,pqrl->pqrs", mo_coeff, twordm)
 
         # Calculate fragment expectation value
-        fragment_energy_one_rdm = 0.25 * np.einsum("ij,ij->", one_rdm[: norb, :], fock[: norb, :] + oneint[: norb, :]) \
-                                  + 0.25 * np.einsum("ij,ij->", one_rdm[:, : norb], fock[:, : norb] + oneint[:, : norb])
+        fragment_energy_onerdm = np.einsum("ij,ij->", onerdm[: norb, :], fock[: norb, :] + oneint[: norb, :]) \
+            + np.einsum("ij,ij->", onerdm[:, : norb], fock[:, : norb] + oneint[:, : norb])
+        fragment_energy_onerdm *= 0.25
 
-        fragment_energy_twordm = 0.125 * np.einsum("ijkl,ijkl->", twordm[: norb, :, :, :], twoint[: norb, :, :, :]) \
-                                 + 0.125 * np.einsum("ijkl,ijkl->", twordm[:, : norb, :, :], twoint[:, : norb, :, :]) \
-                                 + 0.125 * np.einsum("ijkl,ijkl->", twordm[:, :, : norb, :], twoint[:, :, : norb, :]) \
-                                 + 0.125 * np.einsum("ijkl,ijkl->", twordm[:, :, :, : norb], twoint[:, :, :, : norb])
+        fragment_energy_twordm = np.einsum("ijkl,ijkl->", twordm[: norb, :, :, :], twoint[: norb, :, :, :]) \
+            + np.einsum("ijkl,ijkl->", twordm[:, : norb, :, :], twoint[:, : norb, :, :]) \
+            + np.einsum("ijkl,ijkl->", twordm[:, :, : norb, :], twoint[:, :, : norb, :]) \
+            + np.einsum("ijkl,ijkl->", twordm[:, :, :, : norb], twoint[:, :, :, : norb])
+        fragment_energy_twordm *= 0.125
 
-        fragment_energy = fragment_energy_one_rdm + fragment_energy_twordm
+        fragment_energy = fragment_energy_onerdm + fragment_energy_twordm
 
-        return fragment_energy, total_energy_rdm, one_rdm
+        return fragment_energy, onerdm
+
+    def _compute_energy_unrestricted(self, dmet_fragment, onerdm, twordm):
+        """Calculate the fragment energy (unrestricted mean-field).
+
+        Args:
+            dmet_fragment (SecondQuantizedDMETFragment): Self-explanatory.
+            onerdm (numpy.array): one-particle reduced density matrix (float).
+            twordm (numpy.array): two-particle reduced density matrix (float).
+
+        Returns:
+            float64: Fragment energy (fragment_energy).
+            numpy.array: One-particle (alpha) RDM for a fragment (onerdm_a, float64).
+            numpy.array: One-particle (beta) RDM for a fragment (onerdm_b, float64).
+        """
+
+        fock = dmet_fragment.fock
+        t_list = dmet_fragment.t_list
+        oneint = dmet_fragment.one_ele
+        twoint = dmet_fragment.two_ele
+        mo_coeff = dmet_fragment.mean_field.mo_coeff
+
+        norb = t_list[0]
+
+        # Calculate the one- and two- RDM for DMET energy calculation (Transform to AO basis)
+        onerdm_a, onerdm_b = onerdm
+
+        onerdm_a = mo_coeff[0] @ onerdm_a @ mo_coeff[0].T
+        onerdm_b = mo_coeff[1] @ onerdm_b @ mo_coeff[1].T
+
+        twordm_aa, twordm_ab, twordm_bb = twordm
+
+        twordm_aa = np.einsum("pi,ijkl->pjkl", mo_coeff[0], twordm_aa)
+        twordm_aa = np.einsum("qj,pjkl->pqkl", mo_coeff[0], twordm_aa)
+        twordm_aa = np.einsum("rk,pqkl->pqrl", mo_coeff[0], twordm_aa)
+        twordm_aa = np.einsum("sl,pqrl->pqrs", mo_coeff[0], twordm_aa)
+
+        twordm_ab = np.einsum("pi,ijkl->pjkl", mo_coeff[0], twordm_ab)
+        twordm_ab = np.einsum("qj,pjkl->pqkl", mo_coeff[0], twordm_ab)
+        twordm_ab = np.einsum("rk,pqkl->pqrl", mo_coeff[1], twordm_ab)
+        twordm_ab = np.einsum("sl,pqrl->pqrs", mo_coeff[1], twordm_ab)
+
+        twordm_bb = np.einsum("pi,ijkl->pjkl", mo_coeff[1], twordm_bb)
+        twordm_bb = np.einsum("qj,pjkl->pqkl", mo_coeff[1], twordm_bb)
+        twordm_bb = np.einsum("rk,pqkl->pqrl", mo_coeff[1], twordm_bb)
+        twordm_bb = np.einsum("sl,pqrl->pqrs", mo_coeff[1], twordm_bb)
+
+        # Calculate fragment expectation value
+        fragment_energy_one = np.einsum("ij,ij->", onerdm_a[: norb, :], fock[: norb, :] + oneint[: norb, :]) \
+            + np.einsum("ij,ij->", onerdm_a[:, : norb ], fock[:, : norb] + oneint[:, : norb]) \
+            + np.einsum("ij,ij->", onerdm_b[: norb, :], fock[: norb, :] + oneint[: norb, :]) \
+            + np.einsum("ij,ij->", onerdm_b[:, : norb ], fock[:, : norb] + oneint[:, : norb])
+        fragment_energy_one *= 0.25
+
+        fragment_energy_two = np.einsum("ijkl,ijkl->", twordm_aa[: norb, :, :, :], twoint[: norb, :, :, :]) \
+            + np.einsum("ijkl,ijkl->", twordm_aa[:, : norb, :, :], twoint[:, : norb, :, :]) \
+            + np.einsum("ijkl,ijkl->", twordm_aa[:, :, : norb, :], twoint[:, :, : norb, :]) \
+            + np.einsum("ijkl,ijkl->", twordm_aa[:, :, :, : norb], twoint[:, :, :, : norb]) \
+            + np.einsum("ijkl,ijkl->", twordm_ab[: norb, :, :, :], twoint[: norb, :, :, :]) \
+            + np.einsum("ijkl,ijkl->", twordm_ab[:, : norb, :, :], twoint[:, : norb, :, :]) \
+            + np.einsum("ijkl,ijkl->", twordm_ab[:, :, : norb, :], twoint[:, :, : norb, :]) \
+            + np.einsum("ijkl,ijkl->", twordm_ab[:, :, :, : norb], twoint[:, :, :, : norb]) \
+            + np.einsum("klij,ijkl->", twordm_ab[:, :, : norb, :], twoint[: norb, :, :, :]) \
+            + np.einsum("klij,ijkl->", twordm_ab[:, :, :, : norb], twoint[:, : norb, :, :]) \
+            + np.einsum("klij,ijkl->", twordm_ab[: norb, :, :, :], twoint[:, :, : norb, :]) \
+            + np.einsum("klij,ijkl->", twordm_ab[:, : norb, :, :], twoint[:, :, :, : norb]) \
+            + np.einsum("ijkl,ijkl->", twordm_bb[: norb, :, :, :], twoint[: norb, :, :, :]) \
+            + np.einsum("ijkl,ijkl->", twordm_bb[:, : norb, :, :], twoint[:, : norb, :, :]) \
+            + np.einsum("ijkl,ijkl->", twordm_bb[:, :, : norb, :], twoint[:, :, : norb, :]) \
+            + np.einsum("ijkl,ijkl->", twordm_bb[:, :, :, : norb], twoint[:, :, :, : norb])
+        fragment_energy_two *= 0.125
+
+        fragment_energy = fragment_energy_one + fragment_energy_two
+
+        return fragment_energy, onerdm_a, onerdm_b
 
     def _default_optimizer(self, func, var_params):
         """Function used as a default optimizer for DMET when user does not
