@@ -1,4 +1,4 @@
-# Copyright 2021 Good Chemistry Company.
+# Copyright 2023 Good Chemistry Company.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -32,6 +32,7 @@ class ADAPTAnsatz(Ansatz):
     Attributes:
         n_spinorbitals (int): Number of spin orbitals in a given basis.
         n_electrons (int): Number of electrons.
+        spin (int): Spin of system.
         operators (list of QubitOperator): List of operators to consider at the
             construction step. Can be useful for restarting computation.
         ferm_operators (list of FermionOperator): Same as operators, but in
@@ -43,7 +44,7 @@ class ADAPTAnsatz(Ansatz):
         circuit (Circuit): Quantum circuit defined by a list of Gates.
     """
 
-    def __init__(self, n_spinorbitals, n_electrons, ansatz_options=None):
+    def __init__(self, n_spinorbitals, n_electrons, spin, ansatz_options=None):
         default_options = {"operators": list(), "ferm_operators": list(),
                            "mapping": "jw", "up_then_down": False,
                            "reference_state": "HF"}
@@ -60,6 +61,7 @@ class ADAPTAnsatz(Ansatz):
 
         self.n_spinorbitals = n_spinorbitals
         self.n_electrons = n_electrons
+        self.spin = spin
 
         self.var_params = None
         self.circuit = None
@@ -105,7 +107,8 @@ class ADAPTAnsatz(Ansatz):
     def prepare_reference_state(self):
         """Prepare a circuit generating the HF reference state."""
         if self.reference_state.upper() == "HF":
-            return get_reference_circuit(n_spinorbitals=self.n_spinorbitals, n_electrons=self.n_electrons, mapping=self.mapping, up_then_down=self.up_then_down)
+            return get_reference_circuit(n_spinorbitals=self.n_spinorbitals, n_electrons=self.n_electrons,
+                                         mapping=self.mapping, up_then_down=self.up_then_down, spin=self.spin)
         else:
             return Circuit(n_qubits=get_qubit_number(self.mapping, self.n_spinorbitals))
 
