@@ -1,4 +1,4 @@
-# Copyright 2021 Good Chemistry Company.
+# Copyright 2023 Good Chemistry Company.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -88,8 +88,8 @@ class ILC(Ansatz):
         self.molecule = molecule
         if isinstance(self.molecule, SecondQuantizedMolecule):
             self.n_spinorbitals = self.molecule.n_active_sos
-            self.n_electrons = self.molecule.n_electrons
-            self.spin = self.molecule.spin
+            self.n_electrons = self.molecule.n_active_electrons
+            self.spin = self.molecule.active_spin
         elif isinstance(self.molecule, dict):
             self.n_spinorbitals = self.molecule["n_spinorbitals"]
             self.n_electrons = self.molecule["n_electrons"]
