@@ -1,4 +1,4 @@
-# Copyright 2021 Good Chemistry Company.
+# Copyright 2023 Good Chemistry Company.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -86,7 +86,7 @@ class QMF(Ansatz):
             raise ValueError("The total number of spin-orbitals should be even.")
 
         self.n_orbitals = self.n_spinorbitals // 2
-        self.spin = molecule.spin
+        self.spin = molecule.active_spin
         self.fermi_ham = self.molecule.fermionic_hamiltonian
         self.n_electrons = self.molecule.n_active_electrons
 
