@@ -1,4 +1,4 @@
-# Copyright 2021 Good Chemistry Company.
+# Copyright 2023 Good Chemistry Company.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -319,8 +319,12 @@ def get_uprep_uselect(qu_op: QubitOperator, control: Union[int, List[int]] = Non
     max_qu_op = count_qubits(qu_op)
     for term, coeff in qu_op.terms.items():
         acoeff = np.abs(coeff)
-        vector += [acoeff]
-        unitaries += [QubitOperator(term, -coeff / acoeff)]
+        if acoeff > 1.e-8:
+            vector += [acoeff]
+            unitaries += [QubitOperator(term, -coeff / acoeff)]
+        else:
+            vector += [0]
+            unitaries += [QubitOperator((), 1)]
 
     # create U_{prep} from sqrt of coefficients
     vector = np.array(vector)
