@@ -1,4 +1,4 @@
-# Copyright 2021 Good Chemistry Company.
+# Copyright 2023 Good Chemistry Company.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -16,12 +16,11 @@ import unittest
 import os
 
 import numpy as np
-from openfermion.chem.molecular_data import spinorb_from_spatial
 from openfermion.utils import load_operator
 
 from tangelo import SecondQuantizedMolecule
 from tangelo.molecule_library import mol_H2_sto3g, xyz_H2O
-from tangelo.toolboxes.molecular_computation.molecule import atom_string_to_list, spatial_from_spinorb
+from tangelo.toolboxes.molecular_computation.molecule import atom_string_to_list
 
 # For openfermion.load_operator function.
 pwd_this_test = os.path.dirname(os.path.abspath(__file__))
@@ -54,16 +53,6 @@ class MoleculeUtilitiesTest(unittest.TestCase):
         """Verify conversion from string to list format for atom coordinates."""
         assert(atom_string_to_list(H2_string) == H2_list)
 
-    def test_spatial_from_spinorb(self):
-        """Test the conversion from spinorbitals to MO coefficients."""
-        _, one_body_mos, two_body_mos = mol_H2_sto3g.get_integrals()
-
-        one_body_sos, two_body_sos = spinorb_from_spatial(one_body_mos, two_body_mos)
-        one_body_mos_recomputed, two_body_mos_recomputed = spatial_from_spinorb(one_body_sos, two_body_sos)
-
-        np.testing.assert_array_almost_equal(one_body_mos, one_body_mos_recomputed)
-        np.testing.assert_array_almost_equal(two_body_mos, two_body_mos_recomputed)
-
 
 class SecondQuantizedMoleculeTest(unittest.TestCase):
 
@@ -88,71 +77,6 @@ class SecondQuantizedMoleculeTest(unittest.TestCase):
         assert(molecule.n_electrons == 2)
         atom_list_close(molecule.xyz, H2_list, 1.e-14)
 
-    def test_freezing_orbitals(self):
-        """Verify freezing orbitals functionalities."""
-
-        no_freezing = SecondQuantizedMolecule(H2O_list, 0, 0, "sto-3g", frozen_orbitals=None)
-        assert(no_freezing.active_occupied == [0, 1, 2, 3, 4])
-        assert(no_freezing.active_virtual == [5, 6])
-        assert(no_freezing.frozen_occupied == [])
-        assert(no_freezing.frozen_virtual == [])
-
-        freeze_with_int = SecondQuantizedMolecule(H2O_list, frozen_orbitals=3)
-        assert(freeze_with_int.active_occupied == [3, 4])
-        assert(freeze_with_int.active_virtual == [5, 6])
-        assert(freeze_with_int.frozen_occupied == [0, 1, 2])
-        assert(freeze_with_int.frozen_virtual == [])
-
-        freeze_with_list = SecondQuantizedMolecule(H2O_list, frozen_orbitals=[0, 1, 2, 6])
-        assert(freeze_with_list.active_occupied == [3, 4])
-        assert(freeze_with_list.active_virtual == [5])
-        assert(freeze_with_list.frozen_occupied == [0, 1, 2])
-        assert(freeze_with_list.frozen_virtual == [6])
-
-    def test_freezing_empty(self):
-        """Verify freezing orbitals empty input."""
-
-        # None should result in the same as nothing.
-        none_as_frozen = SecondQuantizedMolecule(H2O_list, frozen_orbitals=None)
-        assert(none_as_frozen.active_occupied == [0, 1, 2, 3, 4])
-        assert(none_as_frozen.active_virtual == [5, 6])
-        assert(none_as_frozen.frozen_occupied == [])
-        assert(none_as_frozen.frozen_virtual == [])
-
-        # An empty list should result in the same as nothing.
-        empty_as_frozen = SecondQuantizedMolecule(H2O_list, frozen_orbitals=[])
-        assert(empty_as_frozen.active_occupied == [0, 1, 2, 3, 4])
-        assert(empty_as_frozen.active_virtual == [5, 6])
-        assert(empty_as_frozen.frozen_occupied == [])
-        assert(empty_as_frozen.frozen_virtual == [])
-
-    def test_freezing_type_exception(self):
-        """Verify freezing orbitals exceptions."""
-
-        # Cases where the input is wrong type.
-        with self.assertRaises(TypeError):
-            SecondQuantizedMolecule(H2O_list, frozen_orbitals="3")
-        with self.assertRaises(TypeError):
-            SecondQuantizedMolecule(H2O_list, frozen_orbitals=3.141592)
-        with self.assertRaises(TypeError):
-            SecondQuantizedMolecule(H2O_list, frozen_orbitals=[0, 1, 2.2222, 3, 4, 5])
-
-    def test_no_active_electron(self):
-        """Verify if freezing all active orbitals fails."""
-
-        # Cases where no active electron are remaining.
-        with self.assertRaises(ValueError):
-            SecondQuantizedMolecule(H2O_list, frozen_orbitals=5)
-        with self.assertRaises(ValueError):
-            SecondQuantizedMolecule(H2O_list, frozen_orbitals=[0, 1, 2, 3, 4, 5])
-
-    def test_no_active_virtual(self):
-        """Verify if freezing all virtual orbitals fails."""
-
-        # Cases where no active virtual orbitals are remaining.
-        with self.assertRaises(ValueError):
-            SecondQuantizedMolecule(H2O_list, frozen_orbitals=[5, 6])
-
     def test_all_active_orbitals_occupied_but_some_not_fully(self):
         """Verify that having all active orbitals occupied but only partially occupied is permitted"""
         try:
@@ -228,6 +152,17 @@ class SecondQuantizedMoleculeTest(unittest.TestCase):
         with self.assertRaises(AssertionError):
             molecule.mo_coeff = bad_dummy_mo_coeff
 
+        molecule = SecondQuantizedMolecule(H2_list, 0, 0, "sto-3g", uhf=True)
+
+        # Should work.
+        dummy_mo_coeff = [np.ones((2, 2))]*2
+        molecule.mo_coeff = dummy_mo_coeff
+
+        # Should raise an AssertionError.
+        bad_dummy_mo_coeff = [np.ones((3, 3))]*2
+        with self.assertRaises(AssertionError):
+            molecule.mo_coeff = bad_dummy_mo_coeff
+
 
 if __name__ == "__main__":
     unittest.main()
